import os
import glob

from invoke import Collection, task

from . import py, test


collections = [py, test]

ns = Collection()
for c in collections:
    ns.add_collection(c)

ns.configure(dict(
    project='kazoo-sdk',
    pwd=os.getcwd()
))



"""
kzconfig.kubectl
~~~~~

Kazoo config library.


"""

class KubeCtlCommandBase:
    def __init__(self, context):
        self.context = context

    def _build_command(self, *args):
        args = ['kubectl', self.command] + list(args)
        return ' '.join(args)


class KubeCtlConfig(KubeCtlCommandBase):
    command = 'config'

    def set_credentials(self, name='admin', token=''):
        args = ['set-credentials', name]
        if token:
            args.append('--token={}'.format(token))
        return self._build_command(*args)

    def set_context(self, name='local', cluster='local', user='admin',
                    namespace=''):
        args = ['set-context', name]
        kwargs = dict(cluster=cluster, user=user, namespace=namespace)
        for k, v in kwargs.items():
            args.append('--{}={}'.format(k, v))
        return self._build_command(*args)


class KubeCtl:
    def __init__(self, context):
        self.context = context
        self.config = KubeCtlConfig(self)



"""
kzconfig.cli.sup
~~~~~

Kazoo config library.

A sup cli command for invoking sup against a remote kazoo container.
"""

import click

from ..context import Context


@click.command()
@click.argument('module')
@click.argument('function')
@click.argument('args', nargs=-1)
def main(module, function, args):
    context = Context()
    return click.echo(context.sup.sup(module, function, *args))



from dnsimple import DNSimple


class DNS:
    def __init__(self, context, domain='telephone.org'):
        self.context = context
        self.domain = domain
        creds = self.context.secrets['dns.dnsimple']

        self.api = DNSimple(
            username=creds['email'],
            password=creds['password']
        )

    def get(self, kind='A', name='', content='', **kwargs):
        recs = self.api.records(self.domain)
        recs = [r['record'] for r in recs]
        if kind:
            recs = [r for r in recs if r['record_type'] == kind.upper()]
        if name:
            recs = [r for r in recs if r['name'].lower() == name.lower()]
        if content:
            recs = [r for r in recs if r['content'] == content]
        return recs


    def add(self, kind='A', name='', content='', **kwargs):
        recs = self.get(kind, name, content, **kwargs)
        if recs:
            print('record exists, do not recreate')
            return False
        if not recs:
            print('record does not exist, creating')
            data = dict(
                record_type=kind.upper(),
                name=name,
                content=content
            )
            data.update(**kwargs)
            return self.api.add_record(self.domain, data)

    def delete(self, kind='A', name='', content='', **kwargs):
        recs = self.get(kind, name, content, **kwargs)
        for record_id in [rec['id'] for rec in recs]:
            self.api.delete_record(self.domain, record_id)
        print('{} records deleted'.format(len(recs)))



import pyrkube


class Kube:
    def __init__(self, context):
        self.context = context
        try:
            self.api = pyrkube.KubeAPIClient()
        except pyrkube.KubeConfigNotFound:
            pass



"""
kzconfig.kazoo
~~~~~

Kazoo config library.


"""

import requests
import kazoo


class Kazoo:
    def __init__(self, context):
        self.context = context
        try:
            master_acct = self.context.secrets['master-account']
            env = self.context.configs['environment']
            self.api = kazoo.Client(
                username=master_acct['user'],
                password=master_acct['pass'],
                account_name=master_acct['name'],
                base_url=env['uri.crossbar']
            )
            try:
                self.api.authenticate()
            except requests.ConnectionError:
                pass
        except AttributeError:
            pass



"""
kzconfig.couch
~~~~~

Kazoo config library.

"""

import json
from urllib.parse import urlparse, unquote

import couchdb

from . import util


class CouchDBDoc:
    def __init__(self, parent):
        self.parent = parent

    def create(self, db, doc):
        if isinstance(doc, str):
            doc = json.loads(doc)
        doc_id = doc['_id']

        db = self.parent.api[db]
        if doc_id in db:
            old_doc = db[doc_id]
            doc['_rev'] = old_doc.rev
        return db.save(doc)


class CouchDB:
    def __init__(self, context):
        env = context.configs['environment']
        creds = context.secrets['couchdb']

        p = urlparse(env['uri.couchdb'])
        self.api = couchdb.Server(util.join_url(
            p.scheme, creds['user'], creds['pass'], p.netloc
        ))
        self.doc = CouchDBDoc(self)

    def acct_db(self, acct_id):
        db = self.api['accounts']
        doc = db[acct_id]
        db_name = doc['pvt_account_db']
        print(db_name)
        return api[unquote(db_name)]


#
# def get_db_for(acct_id):
#     api = context.couchdb
#
#     db = api['accounts']
#     doc = db[acct_id]
#     db_name = doc['pvt_account_db']
#     print(db_name)
#     return api[unquote(db_name)]



import re
from setuptools import setup, find_packages


with open('kzconfig/__init__.py', 'rt') as fd:
    version = re.search(r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]',
                        fd.read(), re.MULTILINE).group(1)

if not version:
    raise RuntimeError('Cannot find version information')

with open('README.md') as fd:
    long_description = fd.read()


setup(
    name='kzconfig',
    version=version,
    description="Convenience library for configuring Kazoo & it's components",
    long_description=long_description,
    author='Joe Black',
    author_email='me@joeblack.nyc',
    url='https://github.com/telephoneorg/kzconfig',
    download_url=(
        'https://github.com/telephoneorg/kzconfig/tarball/v%s' % version),
    license='Apache 2.0',
    zip_safe=False,
    packages=find_packages(),
    package_data={'': ['LICENSE']},
    install_requires=[
        'requests',
        'click',
        'couchdb',
        'pyrkube>=0.2.5',
        'kazoo-sdk>=0.2.3',
        'dnsimple',
        'dnspython'
    ],
    entry_points=dict(
        console_scripts=[
            'sup = kzconfig.cli.sup:main',
            'install-kubectl = kzconfig.cli.kubectl:main'
        ]
    ),
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Operating System :: OS Independent',
        'Natural Language :: English',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3 :: Only',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Software Development',
        'Topic :: System',
        'Topic :: System :: Systems Administration'
    ]
)



"""
kzconfig
~~~~~

Kazoo config library.

Kazoo configuration library for configuring kazoo within a kubernetes cluster.
This library includes a sup cli command for invoking sup against a remote
kazoo container.
"""

import json
from collections import OrderedDict

from . import meta, util, context, couch, kazoo, sup, dns, cli
from .context import Context

__title__ = 'kzconfig'
__version__ = '0.3.2'
__author__ = "Joe Black <me@joeblack.nyc>"
__license__ = 'Apache 2.0'
__copyright__ = 'Copyright 2017 Joe Black'


# monkeypatch json to preserve order of keys
json._default_decoder = json.JSONDecoder(
    object_hook=None,
    object_pairs_hook=OrderedDict
)
